/* Terraform - (C) 1997-1999 Robert Gasch (r.gasch@chello.nl)
 *  - http://212.187.12.197/RNG/terraform/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#include "TFDialogPrintOptions.h"
#include "GlobalSanityCheck.h"
#include "GlobalTrace.h"
#include "GuiBufferedDrawingArea.h"
#include "GuiColormap.h"
#include "MathTrig.h"
#include "TFWindowHandler.h"			// circular 
#include "PrintOptions.h"


// Paper definitions stollen from gnome-libs.
// All measurements are in centimetres. 
static const struct _dia_paper_metrics 
	{
	gchar *paper;
	gdouble pswidth, psheight;
	gdouble tmargin, bmargin, lmargin, rmargin;
	} 
	paper_metrics[] = 
		{
		{ "A3", 29.7, 42.0, 2.8222, 2.8222, 2.8222, 2.8222 },
		{ "A4", 21.0, 29.7, 2.8222, 2.8222, 2.8222, 2.8222 },
		{ "A5", 14.85, 21.0, 2.8222, 2.8222, 2.8222, 2.8222 },
		{ "B4", 25.7528, 36.4772, 2.1167, 2.1167, 2.1167, 2.1167 },
		{ "B5", 17.6389, 25.0472, 2.8222, 2.8222, 2.8222, 2.8222 },
		{ "B5-Japan", 18.2386, 25.7528, 2.8222, 2.8222, 2.8222, 2.8222 },
		{ "Letter", 21.59, 27.94, 2.54, 2.54, 2.54, 2.54 },
		{ "Legal", 21.59, 35.56, 2.54, 2.54, 2.54, 2.54 },
		{ "Half-Letter", 21.59, 14.0, 2.54, 2.54, 2.54, 2.54 },
		{ "Executive", 18.45, 26.74, 2.54, 2.54, 2.54, 2.54 },
		{ "Tabloid", 28.01, 43.2858, 2.54, 2.54, 2.54, 2.54 },
		{ "Monarch", 9.8778, 19.12, 0.3528, 0.3528, 0.3528, 0.3528 },
		{ "SuperB", 29.74, 43.2858, 2.8222, 2.8222, 2.8222, 2.8222 },
		{ "Envelope-Commercial", 10.5128, 24.2, 0.1764, 0.1764, 0.1764, 0.1764 },
		{ "Envelope-Monarch", 9.8778, 19.12, 0.1764, 0.1764, 0.1764, 0.1764 },
		{ "Envelope-DL", 11.0, 22.0, 0.1764, 0.1764, 0.1764, 0.1764 },
		{ "Envelope-C5", 16.2278, 22.9306, 0.1764, 0.1764, 0.1764, 0.1764 },
		{ "EuroPostcard", 10.5128, 14.8167, 0.1764, 0.1764, 0.1764, 0.1764 },
		{ NULL, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 }
		};

enum {
	CHANGED,
	FITTOPAGE,
	LAST_SIGNAL
	};


/*
 *  TFDialogPrintOptions: provide a GUI interface to the print options
 * 	stored in PrintOptions.h
 */
TFDialogPrintOptions::TFDialogPrintOptions ()
	        : GuiDialogYN ("Terraform Print Options Dialog", "Print Options Parameters", 'N')
			
{
	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "+++ TFDialogPrintOptions\n");

	d_paperSelected = 1;
	d_col = new GdkColor ();
	d_colB = new GdkColor ();
	d_colB->red = d_colB->green = d_colB->blue = 0;
	d_colW = new GdkColor ();
	d_colW->red = d_colW->green = d_colW->blue = 255;
	d_glstPaperSize = d_glstColor = NULL;
	d_btnOrientLandscape = d_btnOrientPortrait = NULL;
	d_tblMargins = NULL;

	buildDialogWindow ();

	// connect the signal handlers 
	connect_to_method (d_btnOrientPortrait->toggled, this, &TFDialogPrintOptions::orient_changed);
	connect_to_method (d_spbtnMarginT->changed, this, &TFDialogPrintOptions::margin_changed);
	connect_to_method (d_spbtnMarginB->changed, this, &TFDialogPrintOptions::margin_changed);
	connect_to_method (d_spbtnMarginL->changed, this, &TFDialogPrintOptions::margin_changed);
	connect_to_method (d_spbtnMarginR->changed, this, &TFDialogPrintOptions::margin_changed);
	connect_to_method (d_spbtnScaling->changed, this, &TFDialogPrintOptions::scale_changed);
	connect_to_method (d_btnFitToPage->pressed, this, &TFDialogPrintOptions::fitToPagePressed);
}


/*
 *  destructor: clean up 
 */
TFDialogPrintOptions::~TFDialogPrintOptions ()
{
	delete d_col;
	delete d_colB;
	delete d_colW;
	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "--- TFDialogPrintOptions\n");
}


/*
 *  destructor: clean up 
 */
void TFDialogPrintOptions::init ()
{
	Gtk_MenuItem	*menuitem;
	Gtk_Adjustment	*adj;
	Gtk_Pixmap	*pmap;
	string		*s;
	gint i;
	char		buf[256];
	GdkColor	col;

	d_colorMap = new GuiColormap (this->get_toplevel(), 2);
	d_colorMap->addNamedColor (&col, "black");
	d_colorMap->addNamedColor (&col, "white");

	d_tblMain = new Gtk_Table (3, 2);
	d_tblMain->set_row_spacings (5);
	d_tblMain->set_col_spacings (5);
	this->p_frmBase->add (*d_tblMain);

	// 
	// paper size 
	// 
	d_frmPaperSize = new Gtk_Frame ("Paper Size");
	d_tblMain->attach (*d_frmPaperSize, 0, 1, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
	d_frmPaperSize->show ();

	d_vbPaperSize = new Gtk_VBox (FALSE, 5);
	d_vbPaperSize->set_border_width (5);
	d_frmPaperSize->add (*d_vbPaperSize);
	d_vbPaperSize->show ();

	d_optMenuPaperSize = new Gtk_OptionMenu ();
	d_vbPaperSize->pack_start (*d_optMenuPaperSize, TRUE, FALSE, 0);

	d_menuPaperSize = new Gtk_Menu ();
	for (i=0; paper_metrics[i].paper!=NULL; i++) 
		{
		menuitem = new Gtk_MenuItem (paper_metrics[i].paper);
		menuitem->set_user_data (GINT_TO_POINTER(i));
		connect_to_method (menuitem->activate, this, &TFDialogPrintOptions::paper_size_change);
		//connect_to_method (menuitem->activate, this, &TFDialogPrintOptions::paper_size_change);
		//gtk_signal_connect(GTK_OBJECT(menuitem), "activate",
		//       GTK_SIGNAL_FUNC(paper_size_change), d_tblMain);
		d_menuPaperSize->add (*menuitem);
		menuitem->show ();
		//glst = g_list_append (glst, const_cast<char*>(paper_metrics[i].paper));
		}
	d_menuPaperSize->set_active (d_paperSelected);
	d_optMenuPaperSize->set_menu (*d_menuPaperSize);
	d_optMenuPaperSize->show ();
	
	d_lblPaperSize = new Gtk_Label ("XXX");
	d_vbPaperSize->pack_start (*d_lblPaperSize, TRUE, TRUE, 0);
	d_lblPaperSize->show ();

	// 
	// orientation 
	// 
	d_frmOrientation = new Gtk_Frame ("Orientation");
	d_tblMain->attach (*d_frmOrientation, 1,2, 0,1, GTK_FILL|GTK_EXPAND, GTK_FILL, 0, 0);
	d_frmOrientation->show ();

	d_hbOrientation= new Gtk_HBox (FALSE, 5);
	d_hbOrientation->set_border_width (5);
	d_frmOrientation->add (*d_hbOrientation);
	d_hbOrientation->show ();

	d_btnOrientPortrait = new Gtk_RadioButton ((GSList*)NULL);
	sprintf (buf, "%s/%s", TF_DATADIR, "portrait.xpm");
	s = new string (buf);
	pmap = new Gtk_Pixmap (*this, *s);
	d_btnOrientPortrait->add (*pmap);
	delete s;
	pmap->show ();

	d_hbOrientation->pack_start (*d_btnOrientPortrait, TRUE, TRUE, 0);
	d_btnOrientPortrait->show ();

	d_btnOrientLandscape = new Gtk_RadioButton (d_btnOrientPortrait->group());
	sprintf (buf, "%s/%s", TF_DATADIR, "landscape.xpm");
	s = new string (buf);
	pmap = new Gtk_Pixmap (*this, *s);
	d_btnOrientLandscape->add (*pmap);
	delete s;
	pmap->show ();

	d_hbOrientation->pack_start (*d_btnOrientLandscape, TRUE, TRUE, 0);
	d_btnOrientLandscape->show ();

	// 
	// margins 
	// 
	d_frmMargins = new Gtk_Frame ("Margins");
	d_tblMain->attach (*d_frmMargins, 0, 1, 1, 2, GTK_FILL, GTK_FILL, 0, 0);
	d_frmMargins->show ();

	d_tblMargins = new Gtk_Table (4, 2, FALSE);
	d_tblMargins->set_border_width (5);
	d_tblMargins->set_row_spacings (5);
	d_tblMargins->set_col_spacings (5);
	d_frmMargins->add (*d_tblMargins);
	d_tblMargins->show ();

	d_lblT = new Gtk_Label ("Top:");
	d_lblT->set_alignment (1.0, 0.5);
	d_tblMargins->attach (*d_lblT, 0, 1, 0, 1, GTK_FILL, GTK_FILL|GTK_EXPAND, 0, 0);
	d_lblT->show ();

	adj = new Gtk_Adjustment (1, 0,100, 0.1,10,10);
	d_spbtnMarginT = new Gtk_SpinButton (*adj);
	d_spbtnMarginT->set_digits (2);
	//2, DIA_UNIT_CENTIMETER);
	d_tblMargins->attach (*d_spbtnMarginT, 1,2, 0,1, GTK_FILL|GTK_EXPAND, GTK_FILL|GTK_EXPAND, 0, 0);
	d_spbtnMarginT->show ();

	d_lblB= new Gtk_Label ("Bottom:");
	d_lblB->set_alignment (1.0, 0.5);
	d_tblMargins->attach (*d_lblB, 0, 1, 1, 2, GTK_FILL, GTK_FILL|GTK_EXPAND, 0, 0);
	d_lblB->show ();

	adj = new Gtk_Adjustment (1, 0,100, 0.1,10,10);
	d_spbtnMarginB = new Gtk_SpinButton (*adj);
	d_spbtnMarginB->set_digits (2);
	d_tblMargins->attach (*d_spbtnMarginB, 1,2, 1,2, GTK_FILL|GTK_EXPAND, GTK_FILL|GTK_EXPAND, 0, 0);
	d_spbtnMarginB->show ();

	d_lblL= new Gtk_Label ("Left:");
	d_lblL->set_alignment (1.0, 0.5);
	d_tblMargins->attach (*d_lblL, 0, 1, 2, 3, GTK_FILL, GTK_FILL|GTK_EXPAND, 0, 0);
	d_lblL->show ();

	adj = new Gtk_Adjustment (1, 0,100, 0.1,10,10);
	d_spbtnMarginL = new Gtk_SpinButton (*adj);
	d_spbtnMarginL->set_digits (2);
	d_tblMargins->attach (*d_spbtnMarginL, 1,2, 2,3, GTK_FILL|GTK_EXPAND, GTK_FILL|GTK_EXPAND, 0, 0);
	d_spbtnMarginL->show ();

	d_lblR= new Gtk_Label ("Right:");
	d_lblR->set_alignment (1.0, 0.5);
	d_tblMargins->attach (*d_lblR, 0, 1, 3, 4, GTK_FILL, GTK_FILL|GTK_EXPAND, 0, 0);
	d_lblR->show ();

	adj = new Gtk_Adjustment (1, 0,100, 0.1,10,10);
	d_spbtnMarginR = new Gtk_SpinButton (*adj);
	d_spbtnMarginR->set_digits (2);
	d_tblMargins->attach (*d_spbtnMarginR, 1,2, 3,4, GTK_FILL|GTK_EXPAND, GTK_FILL|GTK_EXPAND, 0, 0);
	d_spbtnMarginR->show ();

	// 
	// Scaling 
	// 
	d_frmScaling = new Gtk_Frame ("Scaling");
	d_tblMain->attach (*d_frmScaling, 0, 1, 2, 3, GTK_FILL, GTK_FILL, 0, 0);
	d_frmScaling->show ();

	d_vbScaling= new Gtk_VBox (FALSE, 5);
	d_vbScaling->set_border_width (5);
	d_frmScaling->add (*d_vbScaling);
	d_vbScaling->show ();

	adj = new Gtk_Adjustment (100,1,10000, 1,10,10);
	d_spbtnScaling = new Gtk_SpinButton (*adj);
	d_vbScaling->pack_start (*d_spbtnScaling, TRUE, TRUE, 0);
	d_spbtnScaling->show ();

	d_btnFitToPage = new Gtk_Button ("Fit to Page");
	d_vbScaling->pack_start (*d_btnFitToPage, TRUE, FALSE, 0);
	d_btnFitToPage->show ();

	// the drawing area 
	d_da = new GuiBufferedDrawingArea ();
	d_da->setSync (1);
	d_tblMain->attach (*d_da, 1,2, 1,3, 
		   GTK_FILL|GTK_EXPAND, GTK_FILL|GTK_EXPAND, 0, 0);
	d_da->show ();
	d_tblMargins->show ();
	d_tblMain->show ();
}


void TFDialogPrintOptions::setPaper (const gchar *paper)
{
	gint	i;

	for (i=0; paper_metrics[i].paper != NULL; i++) 
    		if (!g_strcasecmp(paper_metrics[i].paper, paper))
			break;

	if (paper_metrics[i].paper == NULL)
		i = 1; /* A4 */

	d_optMenuPaperSize->set_history(i);
}


void TFDialogPrintOptions::getMargins ( gfloat *tmargin, gfloat *bmargin,
			    		gfloat *lmargin, gfloat *rmargin)
{
	if (tmargin)
		*tmargin = d_spbtnMarginT->get_value_as_float();
	if (bmargin)
		*bmargin = d_spbtnMarginB->get_value_as_float();
	if (lmargin)
		*lmargin = d_spbtnMarginL->get_value_as_float();
	if (rmargin)
		*rmargin = d_spbtnMarginR->get_value_as_float();
}


void TFDialogPrintOptions::setMargins ( gfloat tmargin, gfloat bmargin,
			    		gfloat lmargin, gfloat rmargin)
{
	d_spbtnMarginT->set_value (tmargin);
	d_spbtnMarginB->set_value (bmargin);
	d_spbtnMarginL->set_value (lmargin);
	d_spbtnMarginR->set_value (rmargin);
}



int TFDialogPrintOptions::getOrientation ()
{
	if (d_btnOrientPortrait->get_active())
		return 0; //DIA_PAGE_ORIENT_PORTRAIT;
	else
		return 1; //DIA_PAGE_ORIENT_LANDSCAPE;
}


void TFDialogPrintOptions::setOrientation (int orient)
{
	switch (orient) 
		{
		case 0: //DIA_PAGE_ORIENT_PORTRAIT:
			d_btnOrientPortrait->set_active (TRUE);
	    		break;
		case 1: //DIA_PAGE_ORIENT_LANDSCAPE:
			d_btnOrientLandscape->set_active (TRUE);
			break;
		}
}


gfloat TFDialogPrintOptions::getScaling ()
{
	return d_spbtnScaling->get_adjustment()->get_value()/100.0;
}


void TFDialogPrintOptions::setScaling (gfloat scaling)
{
//	d_spbtnScaling->
//  GTK_SPIN_BUTTON(self->scaling)->adjustment->value = scaling * 100.0;
//  gtk_adjustment_value_changed(GTK_SPIN_BUTTON(self->scaling)->adjustment);
}


void TFDialogPrintOptions::getEffectiveArea(gfloat *width, gfloat *height)
{
	gfloat 		h, w, scaling;

	if (d_btnOrientPortrait->get_active()) 
		{
		w = paper_metrics[d_paperSelected].pswidth;
		h = paper_metrics[d_paperSelected].psheight;
		} 
	else 
		{
		h = paper_metrics[d_paperSelected].pswidth;
		w = paper_metrics[d_paperSelected].psheight;
		}
	h -= d_spbtnMarginT->get_value_as_float();
	h -= d_spbtnMarginB->get_value_as_float();
	w -= d_spbtnMarginL->get_value_as_float();
	w -= d_spbtnMarginR->get_value_as_float();
	scaling = d_spbtnScaling->get_value_as_float() / 100.0;
	h /= scaling;
	w /= scaling;

	if (width)  
		*width = w;
	if (height) 
		*height = h;
}


void TFDialogPrintOptions::getPaperSize (const gchar *paper,
			       gfloat *width, gfloat *height)
{
	gint 	i;

	for (i = 0; paper_metrics[i].paper != NULL; i++) 
		{
		if (!g_strcasecmp(paper_metrics[i].paper, paper))
			break;
		}

	if (paper_metrics[i].paper == NULL)
		i = 1; 	// A4 
	if (width)
		*width = paper_metrics[i].pswidth;
	if (height)
		*height = paper_metrics[i].psheight;
}

void TFDialogPrintOptions::getDefaultMargins (const gchar *paper,
				    gfloat *tmargin, gfloat *bmargin,
				    gfloat *lmargin, gfloat *rmargin)
{
	gint 	i;

	for (i = 0; paper_metrics[i].paper != NULL; i++) 
		{
		if (!g_strcasecmp(paper_metrics[i].paper, paper))
			break;
		}

	if (paper_metrics[i].paper == NULL)
		i = 1; 	// A4
	if (tmargin)
		*tmargin = paper_metrics[i].tmargin;
	if (bmargin)
		*bmargin = paper_metrics[i].bmargin;
	if (lmargin)
		*lmargin = paper_metrics[i].lmargin;
	if (rmargin)
		*rmargin = paper_metrics[i].rmargin;
}


void TFDialogPrintOptions::fitToPagePressed ()
{
  //gtk_signal_emit(GTK_OBJECT(self), pl_signals[FITTOPAGE]);
}


void TFDialogPrintOptions::sizePage ()
{
	Gtk_MenuItem	*menuitem = d_menuPaperSize->get_active();
	d_paperSelected = (int)(menuitem->get_user_data());
	d_width = d_da->width() - 3;
	d_height = d_da->height() - 3;

	// change to correct metrics 
	if (d_btnOrientPortrait->get_active())
		{
		if (d_width * paper_metrics[d_paperSelected].psheight >
	    	    d_height * paper_metrics[d_paperSelected].pswidth)
			d_width = (int)(d_height * paper_metrics[d_paperSelected].pswidth /
				  paper_metrics[d_paperSelected].psheight);
		else
			d_height = (int)(d_width * paper_metrics[d_paperSelected].psheight /
				   paper_metrics[d_paperSelected].pswidth);
		} 
	else 
		{
		if (d_width * paper_metrics[d_paperSelected].pswidth >
		    d_height * paper_metrics[d_paperSelected].psheight)
			d_width = (int)(d_height * paper_metrics[d_paperSelected].psheight /
				  paper_metrics[d_paperSelected].pswidth);
		else
		    d_height = (int)(d_width * paper_metrics[d_paperSelected].pswidth /
				paper_metrics[d_paperSelected].psheight);
		}

	d_offx = (d_da->width() - d_width - 3) / 2;
	d_offy = (d_da->height() - d_height - 3) / 2;
}


gint TFDialogPrintOptions::redraw (GdkEventExpose *event)
{
	gfloat 		val;
	gint 		num;

	//printf ("redraw ()\n");

	sizePage ();
	d_da->clear ();

	// draw the page image 
	d_da->setColors (d_colorMap->getColor(0));
	d_da->drawRectangle (d_offx+3, d_offy+3, d_width, d_height, TRUE);

	d_da->setColors (d_colorMap->getColor(1));
	d_da->drawRectangle (d_offx, d_offy, d_width, d_height, TRUE);

	d_da->setColors (d_colorMap->getColor(0));
	d_da->drawRectangle (d_offx, d_offy, d_width-1, d_height-1, FALSE);

	// draw margins 
	if (d_btnOrientPortrait->get_active()) 
		{
		val = d_spbtnMarginT->get_value_as_float();
		num = (int)(d_offy + val * d_height /paper_metrics[d_paperSelected].psheight);
		d_da->drawLine (d_offx+1, num, d_offx+d_width-2,num);

		val = d_spbtnMarginB->get_value_as_float();
		num = (int)(d_offy + d_height - val * d_height / paper_metrics[d_paperSelected].psheight);
		d_da->drawLine (d_offx+1, num, d_offx+d_width-2,num);

		val = d_spbtnMarginL->get_value_as_float(); 
		num = (int)(d_offx + val * d_width / paper_metrics[d_paperSelected].pswidth); 
		d_da->drawLine (num, d_offy+1,num,d_offy+d_height-2);

		val = d_spbtnMarginR->get_value_as_float(); 
		num = (int)(d_offx + d_width - val * d_width / paper_metrics[d_paperSelected].pswidth);
		d_da->drawLine (num, d_offy+1,num,d_offy+d_height-2);
  		} 
	else 
		{
		val = d_spbtnMarginT->get_value_as_float();
		num = (int)(d_offy + val * d_height /paper_metrics[d_paperSelected].pswidth);
		d_da->drawLine (d_offx+1, num, d_offx+d_width-2,num);

		val = d_spbtnMarginB->get_value_as_float();
		num = (int)(d_offy + d_height - val * d_height / paper_metrics[d_paperSelected].pswidth);
		d_da->drawLine (d_offx+1, num, d_offx+d_width-2,num);

		val = d_spbtnMarginL->get_value_as_float(); 
		num = (int)(d_offx + val * d_width / paper_metrics[d_paperSelected].psheight);
		d_da->drawLine (num, d_offy+1,num,d_offy+d_height-2);

		val = d_spbtnMarginR->get_value_as_float(); 
		num = (int)(d_offx + d_width - val * d_width / paper_metrics[d_paperSelected].psheight);
		d_da->drawLine (num, d_offy+1,num,d_offy+d_height-2);
		}
	d_da->setColors (d_colorMap->getColor(0));
	return FALSE;
}


//void TFDialogPrintOptions::paper_size_change(GtkMenuItem *item)
void TFDialogPrintOptions::paper_size_change()
{
	gchar 		buf[512];
	GtkAdjustment	*adj;
	float		w, h;

	//printf ("paper_size_change()\n");

	getEffectiveArea (&w, &h);
	//d_da->set_usize (w, h);

	redraw ();

	d_spbtnMarginT->set_value (paper_metrics[d_paperSelected].tmargin);
	d_spbtnMarginB->set_value (paper_metrics[d_paperSelected].bmargin);
	d_spbtnMarginL->set_value (paper_metrics[d_paperSelected].lmargin);
	d_spbtnMarginR->set_value (paper_metrics[d_paperSelected].rmargin);

	if (d_btnOrientPortrait->get_active ())
		{
		// plain Gtk magic to get around object limitation of setting values
		adj = (GtkAdjustment*)(d_spbtnMarginT->get_adjustment()->gtkobj());
		adj->upper = paper_metrics[d_paperSelected].psheight;
		d_spbtnMarginT->set_adjustment(*d_spbtnMarginT->get_adjustment());

		adj = (GtkAdjustment*)(d_spbtnMarginB->get_adjustment()->gtkobj());
		adj->upper = paper_metrics[d_paperSelected].psheight;
		d_spbtnMarginB->set_adjustment(*d_spbtnMarginB->get_adjustment());

		adj = (GtkAdjustment*)(d_spbtnMarginL->get_adjustment()->gtkobj());
		adj->upper = paper_metrics[d_paperSelected].pswidth;
		d_spbtnMarginL->set_adjustment(*d_spbtnMarginL->get_adjustment());

		adj = (GtkAdjustment*)(d_spbtnMarginR->get_adjustment()->gtkobj());
		adj->upper = paper_metrics[d_paperSelected].pswidth;
		d_spbtnMarginR->set_adjustment(*d_spbtnMarginR->get_adjustment());
		} 
	else 
		{
		// plain Gtk magic to get around object limitation of setting values
		adj = (GtkAdjustment*)(d_spbtnMarginT->get_adjustment()->gtkobj());
		adj->upper = paper_metrics[d_paperSelected].pswidth;
		d_spbtnMarginT->set_adjustment(*d_spbtnMarginT->get_adjustment());

		adj = (GtkAdjustment*)(d_spbtnMarginB->get_adjustment()->gtkobj());
		adj->upper = paper_metrics[d_paperSelected].pswidth;
		d_spbtnMarginB->set_adjustment(*d_spbtnMarginB->get_adjustment());

		adj = (GtkAdjustment*)(d_spbtnMarginL->get_adjustment()->gtkobj());
		adj->upper = paper_metrics[d_paperSelected].psheight;
		d_spbtnMarginL->set_adjustment(*d_spbtnMarginL->get_adjustment());

		adj = (GtkAdjustment*)(d_spbtnMarginR->get_adjustment()->gtkobj());
		adj->upper = paper_metrics[d_paperSelected].psheight;
		d_spbtnMarginR->set_adjustment(*d_spbtnMarginR->get_adjustment());
		}

	g_snprintf(buf, sizeof(buf), "%0.3gcm x %0.3gcm",
		   paper_metrics[d_paperSelected].pswidth,
		   paper_metrics[d_paperSelected].psheight);
	string 	s(buf);
	d_lblPaperSize->set_text (s);

	getEffectiveArea (&w, &h);
}


void TFDialogPrintOptions::orient_changed()
{
	GtkAdjustment	*adj = NULL;

	//printf ("orient_changed()\n");

	redraw ();

	if (d_btnOrientPortrait->get_active ())
		{
		// plain Gtk magic to get around object limitation of setting values
		adj = (GtkAdjustment*)(d_spbtnMarginT->get_adjustment()->gtkobj());
		adj->upper = paper_metrics[d_paperSelected].psheight;
		d_spbtnMarginT->set_adjustment(*d_spbtnMarginT->get_adjustment());

		adj = (GtkAdjustment*)(d_spbtnMarginB->get_adjustment()->gtkobj());
		adj->upper = paper_metrics[d_paperSelected].psheight;
		d_spbtnMarginB->set_adjustment(*d_spbtnMarginB->get_adjustment());

		adj = (GtkAdjustment*)(d_spbtnMarginL->get_adjustment()->gtkobj());
		adj->upper = paper_metrics[d_paperSelected].pswidth;
		d_spbtnMarginL->set_adjustment(*d_spbtnMarginL->get_adjustment());

		adj = (GtkAdjustment*)(d_spbtnMarginR->get_adjustment()->gtkobj());
		adj->upper = paper_metrics[d_paperSelected].pswidth;
		d_spbtnMarginR->set_adjustment(*d_spbtnMarginR->get_adjustment());
		} 
	else 
		{
		// plain Gtk magic to get around object limitation of setting values
		adj = (GtkAdjustment*)(d_spbtnMarginT->get_adjustment()->gtkobj());
		adj->upper = paper_metrics[d_paperSelected].pswidth;
		d_spbtnMarginT->set_adjustment(*d_spbtnMarginT->get_adjustment());

		adj = (GtkAdjustment*)(d_spbtnMarginB->get_adjustment()->gtkobj());
		adj->upper = paper_metrics[d_paperSelected].pswidth;
		d_spbtnMarginB->set_adjustment(*d_spbtnMarginB->get_adjustment());

		adj = (GtkAdjustment*)(d_spbtnMarginL->get_adjustment()->gtkobj());
		adj->upper = paper_metrics[d_paperSelected].psheight;
		d_spbtnMarginL->set_adjustment(*d_spbtnMarginL->get_adjustment());

		adj = (GtkAdjustment*)(d_spbtnMarginR->get_adjustment()->gtkobj());
		adj->upper = paper_metrics[d_paperSelected].psheight;
		d_spbtnMarginR->set_adjustment(*d_spbtnMarginR->get_adjustment());
		}
}


void TFDialogPrintOptions::margin_changed()
{
	orient_changed ();
	redraw ();
}


void TFDialogPrintOptions::scale_changed()
{
	int	value = d_spbtnScaling->get_value_as_int ();

	redraw ();
}


int TFDialogPrintOptions::idle_redraw ()
{
	paper_size_change ();
	return 0;
}



/*
 *  buildDialogWindow: build the dialog window from it's components
 */
void TFDialogPrintOptions::buildDialogWindow ()
{
	this->set_usize (400, 350);
	this->set_title (this->p_windowTitle);

	this->get_vbox()->set_border_width (2);
	setupVBox ();

	SanityCheck::bailout ((!this->p_frmBase), "this->p_frmBase==NULL", "TFDialogPrintOptions::buildDialogWindow");

	init ();
	this->fillActionArea ();

	this->show ();

	// interate through all the pending events
	Gtk_Main *gtkMain = Gtk_Main::instance();
		while (gtkMain->events_pending())
			gtkMain->iteration ();

	connect_to_method (Gtk_Main::timeout(150), this, &TFDialogPrintOptions::idle_redraw);
}


/*
 *  buttonCallbackApply: roughen the Height Field
 */
void TFDialogPrintOptions::buttonCallbackYes()
{
	PrintOptions::s_paperSize = paper_metrics[d_paperSelected].paper;
	if (d_btnOrientLandscape->get_active())
		PrintOptions::s_orientation='L';
	else
		PrintOptions::s_orientation='P';
	PrintOptions::s_color='C';

	PrintOptions::s_marginT = d_spbtnMarginT->get_value_as_float();
	PrintOptions::s_marginB = d_spbtnMarginB->get_value_as_float();
	PrintOptions::s_marginL = d_spbtnMarginL->get_value_as_float();
	PrintOptions::s_marginR = d_spbtnMarginR->get_value_as_float();

	PrintOptions::s_paperWidth = paper_metrics[d_paperSelected].pswidth;
	PrintOptions::s_paperWidth = paper_metrics[d_paperSelected].psheight;

	PrintOptions::s_scaleFactor = d_spbtnScaling->get_value_as_float ();

	this->hide ();
}




/*
 *  cmbPaperSizeCallback: process a change in paper size selection 
 */
void TFDialogPrintOptions::cmbPaperSizeCallback ()
{
	string		entry;

	//entry = (d_cmbPaperSize->get_entry())->get_text();

	if (entry == "Custom")
		d_enPaperSize.set_sensitive (TRUE);
	else
		d_enPaperSize.set_sensitive (FALSE);
		
}


